//
//  ViewController.swift
//  MRZSampleSwift
//
//  Created by Filip Siljavski on 13/02/19.
//  Copyright © 2019 Scansolutions. All rights reserved.
//

import UIKit
import MRZScannerSDK

class ViewController: UIViewController, MRZScannerDelegate {
    
    enum MRZScanningMode {
        case SCANNER_MODE_MRZ
        case SCANNER_MODE_MRZ_CONTINUOUS
        case SCANNER_MODE_MRZ_CONTINUOUS_IGNORE_DUPLICATES
        case SCANNER_MODE_ID_IMAGE
        case SCANNER_MODE_PASSPORT_IMAGE
        case SCANNER_MODE_GALLERY
        case SCANNER_MODE_PARTIAL_VIEW
        case SCANNER_MODE_ID_SESSION
        case SCANNER_MODE_ID_IMAGE_FRONT
        case SCANNER_MODE_HIDDEN_PREVIEW
    }
    
    private var mrzScannerController: MRZScannerController?
    private var currentScanningMode: MRZScanningMode?
    private var continuousController: ContinuousPreviewTable?
    
    @IBOutlet weak var vibrateOnSuccessSwitch: UISwitch!
    @IBOutlet weak var imageView: UIImageView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    @IBAction func startScannerBtnClick(_ sender: Any) {
        startScanner(mode: .SCANNER_MODE_MRZ)
    }
    
    @IBAction func startContinuousScannerBtnClick(_ sender: Any) {
        startScanner(mode: .SCANNER_MODE_MRZ_CONTINUOUS)
    }
    
    @IBAction func continuousScanningIgnoreDuplicatesBtnClick(_ sender: Any) {
        startScanner(mode: .SCANNER_MODE_MRZ_CONTINUOUS_IGNORE_DUPLICATES)
    }
    
    @IBAction func scanFromGalleryBtnClick(_ sender: Any) {
        startScanner(mode: .SCANNER_MODE_GALLERY)
    }
    
    @IBAction func idImageScannerBtnClick(_ sender: Any) {
        startScanner(mode: .SCANNER_MODE_ID_IMAGE)
    }
    
    @IBAction func passportImageScannerBtnClick(_ sender: Any) {
        startScanner(mode: .SCANNER_MODE_PASSPORT_IMAGE)
    }
    
    @IBAction func partialViewScannerBtnClick(_ sender: Any){
        startScanner(mode: .SCANNER_MODE_PARTIAL_VIEW)
    }
    
    @IBAction func idSessionBtnClick(_ sender: Any) {
        startScanner(mode: .SCANNER_MODE_ID_SESSION)
    }

    @IBAction func frontIdImageScannerClicke(_ sender: Any) {
        startScanner(mode: .SCANNER_MODE_ID_IMAGE_FRONT)
    }
    
    @IBAction func hiddenPrevewBtnClick(_ sender: Any) {
        startScanner(mode: .SCANNER_MODE_HIDDEN_PREVIEW)
    }
    
    private func startScanner(mode: MRZScanningMode) {
        if mode == .SCANNER_MODE_HIDDEN_PREVIEW && mrzScannerController != nil {
            mrzScannerController!.closeScanner()
            mrzScannerController = nil
            return
        }
        
        mrzScannerController?.closeScanner()
        
        currentScanningMode = mode
        if mode == .SCANNER_MODE_GALLERY {
            MRZScannerController.scan(fromGallery: self, delegate: self)
            return
        }
        
        mrzScannerController = MRZScannerController()
        MRZScannerController.registerLicense(withKey: "licence_key")
        mrzScannerController!.delegate = self
        
        //Default params
        mrzScannerController!.setContinuousScanningEnabled(false)
        mrzScannerController!.setIgnoreDuplicates(true)
        mrzScannerController!.setScannerType(TYPE_MRZ)
        mrzScannerController!.enableMinimizedView(true)

        let currentViewController = (self.navigationController != nil) ? self.navigationController : self
        currentViewController?.addChild(mrzScannerController!)
        
        //Configure the scanner for the coresponding mode
        switch (mode) {
        case .SCANNER_MODE_MRZ_CONTINUOUS:
            mrzScannerController!.setContinuousScanningEnabled(true)
            mrzScannerController!.setIgnoreDuplicates(false)
            addContinuousTableOverlay(controller: currentViewController)
        case .SCANNER_MODE_MRZ_CONTINUOUS_IGNORE_DUPLICATES:
            mrzScannerController!.setContinuousScanningEnabled(true)
            addContinuousTableOverlay(controller: currentViewController)
        case .SCANNER_MODE_PASSPORT_IMAGE:
            mrzScannerController!.setScannerType(TYPE_DOCUMENT_IMAGE_PASSPORT)
        case .SCANNER_MODE_ID_IMAGE:
            mrzScannerController!.setScannerType(TYPE_DOCUMENT_IMAGE_ID)
        case .SCANNER_MODE_ID_IMAGE_FRONT:
            mrzScannerController!.setScannerType(TYPE_DOCUMENT_IMAGE_ID_FRONT)
        case .SCANNER_MODE_ID_SESSION:
            mrzScannerController!.setScannerType(TYPE_ID_SESSION)
        case .SCANNER_MODE_MRZ:
            //This is the default mode
            break;
        default:
            break;
        }
        
        MRZScannerController.enableVibration(onSuccess: vibrateOnSuccessSwitch.isOn)
        
        if mode == .SCANNER_MODE_PARTIAL_VIEW {
            //Init with partial view
            mrzScannerController!.initUI(currentViewController, partialViewRectPortrait: CGRect(x: 0, y: 0, width: 100, height: 50), partialViewRectLandscape: CGRect(x: 0, y: 0, width: 50, height: 100))
        } else if mode == .SCANNER_MODE_HIDDEN_PREVIEW {
            mrzScannerController!.initUI(currentViewController, partialViewRect: CGRect(x: 0, y: 0, width: 1, height: 1))
        } else {
            mrzScannerController?.initUI(currentViewController)
        }
    }
    
    func addContinuousTableOverlay(controller: UIViewController?) {
        let storyboard = UIStoryboard(name: "Main", bundle: nil)
        continuousController = (storyboard.instantiateViewController(withIdentifier: "CONTINUOUS_PREVIEW_CONTROLLER") as! ContinuousPreviewTable)

        controller?.view.addSubview(continuousController!.view)

        var safeAreaPadding = CGFloat(0)
        
        if #available(iOS 11.0, *) {
            let window = UIApplication.shared.keyWindow
            safeAreaPadding = max(window?.safeAreaInsets.right ?? 0, window?.safeAreaInsets.bottom ?? 0)
        }

        continuousController?.view.bounds = view.frame.inset(by: UIEdgeInsets(top: 65, left: 0, bottom: 65, right: 70 + safeAreaPadding))
        continuousController?.view.frame.origin.x = 0
        controller?.addChild(continuousController!)
    }
    
    func successfulScan(withResult result: MRZResultDataModel!) {
        if currentScanningMode != MRZScanningMode.SCANNER_MODE_MRZ_CONTINUOUS
            && currentScanningMode != .SCANNER_MODE_MRZ_CONTINUOUS_IGNORE_DUPLICATES {
            showAlert(message: result.toReadableString(), firstImage: result.idBack, secondImage: result.idFront)
        } else {
            continuousController?.addItem(itemVal: result.document_number + " - " + result.fullName)
            if continuousController?.view != nil {
                self.view.bringSubviewToFront((continuousController?.view)!)
            }
        }
    }
    
    func successfulDocumentScan(withImageResult resultImage: UIImage!) {
        imageView.image = resultImage
    }
    
    func successfulIdFrontScan(withFullImage fullImage: UIImage!, portrait: UIImage!) {
        showAlert(message: nil, firstImage: fullImage, secondImage: portrait)
    }
    
    func scanImageFailed() {
        let alertController = UIAlertController.init(title: nil, message: "Scan image failed.", preferredStyle: .alert)
        alertController.addAction(UIAlertAction.init(title: "OK", style: .cancel))
        
        self.present(alertController, animated: true, completion: nil)
    }
    
    func scannerWasDismissed() {
        continuousController?.willMove(toParent: nil)
        continuousController?.view.removeFromSuperview()
        continuousController?.removeFromParent()
        continuousController = nil;
    }
    
    func showAlert(message: String?, firstImage: UIImage?, secondImage: UIImage?) {
        let alertController = UIAlertController.init(title: nil, message: (message ?? ""), preferredStyle: .alert)

        alertController.addAction(UIAlertAction.init(title: "Close", style: .cancel) { (_) in
            self.mrzScannerController?.closeScanner()
            self.scannerWasDismissed()
        })
        
        if firstImage != nil {
            alertController.message = "\n\n\n\n\n\n\n\(alertController.message!)" // image padding
            let backImage = UIImageView(frame: CGRect(x: 0, y: 0, width: 240, height: 165))
            backImage.image = firstImage
            backImage.contentMode = .scaleAspectFit
            alertController.view.addSubview(backImage)
        }
        
        if secondImage != nil {
            alertController.message = "\n\n\n\n\n\n\n\n\(alertController.message!)" // image padding
            let frontImage = UIImageView(frame: CGRect(x: 0, y: 165, width: 240, height: 165))
            frontImage.image = secondImage
            frontImage.contentMode = .scaleAspectFit
            alertController.view.addSubview(frontImage)
        }

        if currentScanningMode != .SCANNER_MODE_ID_SESSION {
            alertController.addAction(UIAlertAction.init(title: "Resume", style: .default) { (_) in
                self.mrzScannerController?.resumeScanner()
            })
        }
        
        self.present(alertController, animated: true, completion: nil)
    }
    
}
